// Définition des dimensions de la visualisation.
const width = 800;
const height = 400;
const margin = {top:20, right:20, bottom:20, left:20};

// Création du canevas SVG.
const svg = d3.select("#arbre_svg")
  .attr("width", width)
  .attr("height", height);

// Ajout d'un groupe (g) pour contenir les éléments de la visualisation.
const g = svg.append("g")
  .attr("transform", `translate(${margin.left},${margin.top})`);

// Données représentant la forêt d'arbres.
let data = [
  {
    id: 1,
    name: "Arbre 1",
    accuracy: 0.9,
    children: [
      {
        name: "x1",
        children: [
          { name: "x2" },
          {
            name: "x3",
            children: [
              { name: "x4" },
              { name: "x5" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 2,
    name: "Arbre 2",
    accuracy: 0.4,
    children: [
      {
        name: "x6",
        children: [
          {
            name: "x3",
            children: [
              { name: "x1" },
              { name: "x8" },
            ],
          },
          {
            name: "x7",
            children: [
              { name: "x2" },
              { name: "x4" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 3,
    name: "Arbre 3",
    accuracy: 0.7,
    children: [
      {
        name: "x8",
        children: [
          {
            name: "x4",
            children: [
              { name: "x1" },
              { name: "x3" },
            ],
          },
          {
            name: "x5",
            children: [
              { name: "x4" },
              { name: "x2" },
            ],
          },
        ],
      },
    ],
  },
  {
    id: 4,
    name: "Arbre 4",
    accuracy: 0.8,
    children: [
      {
        name: "x2",
        children: [
          {
            name: "x1",
            children: [
              { name: "x4" },
              { name: "x3" },
              { name: "x7" },
              { name: "x6" },
              { name: "x5" },
            ],
          },
          { name: "x8" },
        ],
      },
    ],
  },
];

// Tri des arbres par accuracy dans l'ordre décroissant
data.sort((a, b) => b.accuracy - a.accuracy);


// Configuration de la disposition de l'arbre.
const treeLayout = d3.tree().size(
  [0.6 * (height - margin.top - margin.bottom), 
  0.6 * (width - margin.left - margin.right)/3.5]);


// Fonction pour générer des arbres hiérarchiques à partir des données.
function generateTree(treeData, treeId) {
  const root = d3.hierarchy(treeData);
  root.each((node, i) => {
    node.id = `${treeId}-${i}`;
  });
  return treeLayout(root);
}

// Fonction pour déterminer la couleur en fonction de l'accuracy.
function getColorByAccuracy(accuracy) {
  if (accuracy >= 0.8) {
    return "red";
  } else if (accuracy >= 0.6) {
    return "blue";
  } else {
    return "gray";
  }
}


// Variables pour ajuster les positions et tailles des arbres.
let xOffset = 0;
let maxHeight = 0;

// Ajout d'un facteur de mise à l'échelle en fonction de la taille des arbres.
const scaleFactor = 1.2;


//calculer le cercle circonscrit
function calculateBoundingCircle(tree) {
  const xCoords = tree.descendants().map(d => d.x);
  const yCoords = tree.descendants().map(d => d.y);
  const minX = Math.min(...xCoords);
  const maxX = Math.max(...xCoords);
  const minY = Math.min(...yCoords);
  const maxY = Math.max(...yCoords);
  const centerX = (minX + maxX) / 2;
  const centerY = (minY + maxY) / 2;
  const radius = Math.sqrt(Math.pow(maxX - minX, 2) + Math.pow(maxY - minY, 2)) / 2;
  return { centerX, centerY, radius };
}

// Conversion des données en arbres hiérarchiques.
data = data.map((treeData, i) => generateTree(treeData, i + 1));

// Fonction pour basculer la classe "toggled" sur les nœuds ayant le même nom.
function toggleSameNodes(d) {
  // Trouver tous les nœuds portant le même nom dans tous les arbres
  const sameNodes = data.flatMap(tree => tree.descendants())
    .filter(node => node.data.name === d.data.name);

  // Basculer la classe "toggled" sur les nœuds trouvés
  svg.selectAll(".node")
    .filter(node => sameNodes.includes(node))
    .classed("toggled", sameNodes[0].data.toggled ? false : true);
}


const boundingCircles = data.map(tree => calculateBoundingCircle(tree));


// Boucle sur chaque arbre pour créer la visualisation.
data.forEach((tree, index) => {
  const treeWidths = tree.descendants().map(d => d.x);
  const maxTreeWidth = Math.max(...treeWidths);
  const treeHeights = tree.descendants().map(d => d.y);
  const maxTreeHeight = Math.max(...treeHeights);

  // Calcul du décalage sur l'axe Y pour centrer chaque arbre.
  const yOffset = (height - maxTreeHeight) / 2;

  // Création d'un groupe (g) pour chaque arbre.
  const treeGroup = g.append("g")
    .attr("transform", `translate(${xOffset}, ${yOffset})`);

  
  
  // Dessiner le cercle circonscrit pour chaque arbre en utilisant la couleur de la racine.
  const boundingCircle = boundingCircles[index];
  const rootColor = getColorByAccuracy(tree.data.accuracy);

  treeGroup.append("circle")
    .attr("class", "bounding-circle")
    .attr("cx", boundingCircle.centerX)
    .attr("cy", boundingCircle.centerY)
    .attr("r", boundingCircle.radius)
    .attr("stroke", rootColor) // Utilisez la couleur de la racine pour le contour du cercle.
    .attr("fill", rootColor) // Utilisez la couleur de la racine pour le remplissage du cercle.
    .attr("opacity", 0.5); // Ajoutez une opacité pour rendre le cercle semi-transparent.


    treeGroup.append("circle")
      .attr("class", "bounding-circle")
      .attr("cx", boundingCircle.centerX)
      .attr("cy", boundingCircle.centerY)
      .attr("r", boundingCircle.radius)
      .attr("stroke", "black")
      .attr("fill", "none");
  
  // Dessiner les nœuds de chaque arbre (sauf la racine)
  const nodes = treeGroup
  .selectAll(".node")
  .data(tree.descendants())
  .enter()
  .data(tree.descendants().filter(d => d.parent))
  .append("g")
    .attr("class", "node")
    .attr("transform", (d) => `translate(${d.x},${d.y})`)
    .attr("class", function(d) {
    if (!d.parent && d.data.accuracy >= 0.8) {
      return "node high-accuracy";
    } else if (!d.parent && d.data.accuracy >= 0.6) {
      return "node medium-accuracy";
    } else if (!d.parent) {
      return "node low-accuracy";
    } else {
      return "node black";
    }
  })
    .on("mouseover", function(d) {
      tooltip.transition()
        .duration(200)
        .style("opacity", .9);
      tooltip.html(d.data.name)
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
    })
    .on("mouseout", function(d) {
      tooltip.transition()
        .duration(500)
        .style("opacity", 0);
    })
    .on("click", function(d) {
      const node = d3.select(this);
      const isToggled = node.classed("toggled");

      // Remove the "toggled" class from all nodes in all trees
      svg.selectAll(".node").classed("toggled", false);

      // Toggle the "toggled" class on the clicked node and same nodes
      node.classed("toggled", !isToggled);
      toggleSameNodes(d);
    });
  
  
  // Dessiner des cercles pour représenter les nœuds.
  nodes.append("circle")
    .attr("r", function(d) {
        return 10;
  })
    .attr("fill", function(d) {
      if (d.data.accuracy >= 0.8) {
        return "red";
      } else if (d.data.accuracy >= 0.6) {
        return "blue";
      } else {
        return "black";
      }
    })
    .attr("data-name", d => d.data.name);


  // Ajout d'une infobulle pour afficher le nom du nœud.
  const tooltip = d3.select("body").append("div")
    .attr("class", "tooltip")
    .style("opacity", 0);

  // Dessiner les liens entre les nœuds.
  const links = treeGroup
    .selectAll(".link")
    .data(tree.links())
    .enter()
    .append("path")
      .attr("class", "link")
      .attr("d", d3.linkVertical().x(d => d.x).y(d => d.y))
      .attr("stroke", "black")
      .attr("fill", "none");

  // Mise à jour des variables de positionnement et de taille.
  xOffset += maxTreeWidth * scaleFactor + margin.right;
  maxHeight = Math.max(maxHeight, maxTreeHeight);
});

// Ajustement de la largeur et de la hauteur de la zone de dessin si nécessaire.
const totalWidth = xOffset + margin.right;
const totalHeight = maxHeight + margin.top + margin.bottom;
if (totalWidth > width) {
  svg.attr("width", totalWidth);
}
if (totalHeight > height) {
  